@@ -619,6 +619,7 @@ dogroup(const char *arg)
     if (g) {
 	group = g; /* global */
 	if (!is_pseudogroup(g)) {
+	    /* regular news group */
 	    if (isinteresting(arg)) {
 		if (debugmode)
 		    syslog(LOG_DEBUG, "marked group %s interesting", arg);
@@ -632,6 +633,7 @@ dogroup(const char *arg)
 		   g->last >= g->first ? g->last - g->first + 1 : 0,
 		   g->first, g->last, g->name);
 	} else {
+	    /* pseudo news group */
 	    if (debugmode)
 		syslog(LOG_DEBUG,
 		       ">211 %lu %lu %lu %s group selected (pseudo article)",
@@ -738,7 +740,10 @@ printlist(const struct newsgroup *g, const int what)
 {
     switch(what) {
 	case 0:
-	    printf("%s %lu %lu y\r\n", g->name, g->last, g->first);
+	    if (is_pseudogroup(g))
+		printf("%s %lu %lu y\r\n", g->name, g->first, g->first);
+	    else
+		printf("%s %lu %lu y\r\n", g->name, g->last, g->first);
 	    break;
 	case 1:
 	    printf("%s\t%s\r\n", g->name, g->desc ? g->desc : "-x-");
@@ -768,8 +773,11 @@ list(struct newsgroup *g, const int what, const char *pattern)
 	}
     } else {
 	g = findgroup(pattern);
-	if (g)
+	if (g) {
 	    printlist(g, what);
+	    if (what == 0 && !is_pseudogroup(g) && isinteresting(pattern))
+		markinterest(pattern);
+	}
     }
 }
 
@@ -1006,6 +1014,11 @@ dopost(void)
     char *suggmid;
     /*@observer@*/ const char *appendheader = NULL;
 
+    if (getenv("LN_REJECT_POST_PRE")) {
+	printf("400 Posting rejected - debug variable LN_REJECT_POST_PRE exists\r\n");
+	return 0;
+    }
+
     do {
 	(void)xsnprintf(outname, sizeof(outname), "%s/out.going/%d-%d-%d",
 		spooldir, (int)getpid(), (int)time(NULL), ++postingno);
@@ -1220,6 +1233,9 @@ dopost(void)
     if (!havefrom)
 	ferrstr = "Missing From: header";
 
+    if (getenv("LN_REJECT_POST_POST"))
+	ferr = 1;
+
     if (havefrom && havesubject && havenewsgroups && !ferr) {
 	if (!err && 0 == chmod(outname, 0644)) {
 	    printf("240 Article posted, now be patient\r\n");
@@ -1236,6 +1252,12 @@ dopost(void)
 
     (void)unlink(outname);
 
+    if (getenv("LN_REJECT_POST_POST")) {
+	printf("400 Posting rejected - debug variable LN_REJECT_POST_POST exists\r\n");
+	syslog(LOG_INFO, ">400 Posting rejected - debug variable LN_REJECT_POST_POST exists\r\n");
+	return 0;
+    }
+
     if (ferrstr) {
 	printf("441 Post rejected, formatting error: %s\r\n", ferrstr);
 	syslog(LOG_INFO, ">441 Post rejected, formatting error: %s", ferrstr);
