@@ -42,6 +42,7 @@ See file COPYING for restrictions on the use of this software.
 #include <errno.h>
 #include <signal.h>
 #include "mysigact.h"
+#include "mastring.h"
 
 static time_t default_expire;
 
@@ -101,6 +102,7 @@ th(const char *mm) {
     strcpy(p, f);
     t = strrchr(p, '/');
     if (!t) {
+	ln_log(LNLOG_SERR, LNLOG_CTOP, "can't find / - internal error");
 	free(m);
 	free(p);
 	return 1;
@@ -213,16 +215,12 @@ dogroup(/*@null@*/ struct newsgroup *g, const char *name, int expdays)
     }
     closedir(d);
 
+    /* update overview info */
+    getxover();
+    freexover();
+
     if (last < first) {
 	if (verbose > 1) printf("%s: empty group\n", name);
-	if (!isinteresting(name)) {
-	    if (chdir(spooldir)) {
-		ln_log(LNLOG_SERR, LNLOG_CGROUP, "cannot chdir(%s): %m", spooldir);
-	    }
-	    if (rmdir(gdir)) {
-		ln_log(LNLOG_SERR, LNLOG_CGROUP, "cannot rmdir(%s): %m", gdir);
-	    }
-	}
 	if (g) g->first = g->last + 1;
 	free(gdir);
 	return;
@@ -236,10 +234,6 @@ dogroup(/*@null@*/ struct newsgroup *g, const char *name, int expdays)
 	       "%s: expire %lu, low water mark %lu, high water mark %lu",
 	       name, (unsigned long)expdays, first, last);
 
-    /* update overview info */
-    getxover();
-    freexover();
-
     /* allocate and clear article array */
     articles = (struct exp *)critmalloc((last - first + 1) * sizeof(struct exp),
 					"Reading articles to expire");
@@ -337,7 +331,7 @@ dogroup(/*@null@*/ struct newsgroup *g, const char *name, int expdays)
 			int relink = 0;
 			const char *t = lookup(p);
 
-			insertmsgid(p, art);
+			insertmsgid(p);
 
 			if (repair == 0) {
 			    /* fast path */
@@ -370,7 +364,7 @@ dogroup(/*@null@*/ struct newsgroup *g, const char *name, int expdays)
 			    {
 				    ln_log(LNLOG_SERR, LNLOG_CGROUP,
 					   "%s: relink of %s <-> %s failed: %m (%s)",
-					   name, p, de->d_name, lookup(p));
+					   name, p, de->d_name, t);
 			    } else {
 				ln_log(LNLOG_SINFO, LNLOG_CARTICLE,
 					"%s: relinked message %s <-> %s", name, p, de->d_name);
@@ -517,10 +511,12 @@ readmids(void)
 
     fd = open(MIDSFILE, O_RDONLY);
     if (fd < 0) {
-	if (errno != ENOENT)
+	if (errno != ENOENT) {
 	    ln_log(LNLOG_SERR, LNLOG_CTOP, "cannot open \"%s\" file: %m",
 		    MIDSFILE);
-	return errno != ENOENT;
+	    return 1;
+	}
+	return 0;
     }
 
     /* delete file early so we don't barf again and again if the file is
@@ -530,24 +526,24 @@ readmids(void)
     buf = critmalloc(bufsiz, "readmids");
 
     while((r = read(fd, &l, sizeof(l))) == (ssize_t)sizeof(l)) {
+	/* length obtained */
 	if (l+1 > bufsiz) {
 	    free(buf);
 	    bufsiz = l + 1;
 	    buf = critmalloc(bufsiz, "readmids");
 	}
 	if ((r = read(fd, buf, l)) < l) {
+	    /* short read */
 	    rc = -1;
 	    break;
 	}
 	buf[l] = '\0';
-	/* sanity check - the length we've written will usually
-	 * contain three null bytes independent of endianness,
-	 * we'll see these here and skip the file */
+	/* sanity check */
 	if (strlen(buf) != (size_t)l) {
 	    rc = -1;
 	    break;
 	}
-	insertmsgid(buf, 1);
+	insertmsgid(buf);
     }
     free(buf);
     (void)close(fd);
@@ -560,6 +556,24 @@ readmids(void)
     return rc;
 }
 
+/* returns 0 for success */
+static int
+cleanmids(void)
+{
+    int n;
+    mastr *s = mastr_new(256);
+
+    for (n = 0; n < 1000; n++) {
+	char buf[4];
+	snprintf(buf, sizeof(buf), "%03d", n); /* safe */
+	mastr_clear(s);
+	mastr_vcat(s, spooldir, "/message.id/", buf, "/", MIDSFILE, NULL);
+	if (log_unlink(mastr_str(s), 1))
+	    return 1;
+    }
+    return 0;
+}
+
 static void
 expiremsgid(void)
 {
@@ -589,7 +603,7 @@ expiremsgid(void)
 	    }
 	}
 
-	if (!nomids)
+	if (nomids == 0)
 	    nomids |= readmids();
 	else
 	    unlink(MIDSFILE); /* ignore errors */
@@ -599,7 +613,14 @@ expiremsgid(void)
 	    continue;
 	while ((de = readdir(d)) != 0) {
 	    if (stat(de->d_name, &st) == 0 && S_ISREG(st.st_mode)) {
-		if (st.st_nlink < 2 || (!nomids && !findmsgid(de->d_name))) {
+		int ul = 0;
+		char *reason = "";
+		if (st.st_nlink < 2) ul = 1, reason = "link count below 1";
+		if (!nomids && !findmsgid(de->d_name)) ul = 1, reason = "not seen in group scan";
+		if (ul) {
+		    if (debugmode)
+			ln_log(LNLOG_SDEBUG, LNLOG_CARTICLE, "unlinking %03d/%s, %s",
+				n, de->d_name, reason);
 		    if (0 == log_unlink(de->d_name, 1)
 			    && de->d_name[0] == '<' /* only count MID files */)
 			deleted++;
@@ -619,10 +640,15 @@ expiremsgid(void)
 			    ln_log(LNLOG_SERR, LNLOG_CARTICLE,
 				    "rehash: cannot move %s%s to %s: %m",
 				    s, de->d_name, t);
-			else
+			else {
+			    char buf[4];
+			    memcpy(buf, t + s_len - 4, 3);
+			    buf[3] = '\0';
+
 			    ln_log(LNLOG_SINFO, LNLOG_CARTICLE,
-				    "rehashed %s from %03d to %.3s", de->d_name,
-				    n, t + s_len - 4);
+				    "rehashed %s from %03d to %s", de->d_name,
+				    n, buf);
+			}
 			log_unlink(de->d_name, 0);
 		    }
 		}
@@ -704,6 +730,12 @@ main(int argc, char **argv)
 	exit(1);
     }
 
+    if (cleanmids()) {
+	ln_log(LNLOG_SERR, LNLOG_CTOP, "Cannot weed out MIDS files, aborting.\n");
+	unlink(lockfile);
+	exit(1);
+    }
+
     readactive();
     if (!active) {
 	ln_log(LNLOG_SWARNING, LNLOG_CTOP, "Reading active file failed. Trying to build my own.");
